-- All the buffs we watch. Only consolidated buffs are watched.
local watch = {}
local db = nil

-- The currently showed texture, if this buff is reapplied we hide the icon instantly.
local currentIcon = nil

local iconFrame = nil
local iconTimer = 0

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

function frame:PLAYER_ENTERING_WORLD()
	if not BuffDropDB then BuffDropDB = {} end
	for k, v in pairs({
		icon = true,
		raid = true,
		icontime = 2,
	}) do
		if type(BuffDropDB[k]) == "nil" then BuffDropDB[k] = v end
	end
	db = BuffDropDB

	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function frame:COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventtype, _, sourceguid, sourcename, sourceflags, _, destguid, destname, destflags, _, ...)
	if self[eventtype] and CombatLog_Object_IsA(destflags, COMBATLOG_FILTER_MINE) then
		self[eventtype](self, ...)
	end
end

function frame:SPELL_AURA_REMOVED(spellid, spellname, spellschool, auratype)
	if watch[spellname] then
		self:AnnounceBuff(spellname)
	end
end

function frame:ENCHANT_REMOVED(enchantname, itemId, itemname)
	if watch[enchantname] then
		self:AnnounceBuff(enchantname)
	end
end

function frame:SPELL_AURA_APPLIED(spellid, spellname, spellschool, auratype)
	if not watch[spellname] then
		local name, _, icon, _, _, duration, _, _, _, shouldConsolidate = UnitAura("player", spellname)
		if name and duration and duration > 60 and shouldConsolidate then
			watch[spellname] = icon
		end
	end
	if currentIcon == spellname then
		self:HideBigIcon()
	end
end
function frame:ENCHANT_APPLIED(enchantname, itemId, item)
	local _, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(itemId)
	if itemTexture then watch[enchantname] = itemTexture end
	if currentIcon == enchantname then
		self:HideBigIcon()
	end
end

local color_red = {r=1, g=0, b=0}
function frame:AnnounceBuff(buff)
	if db.raid then
		RaidNotice_AddMessage(RaidWarningFrame, buff, color_red)
	end
	if db.icon then
		currentIcon = buff
		self:ShowBigIcon(watch[buff])
	end
end

local function onUpdate(self, elapsed)
	iconTimer = iconTimer + elapsed
	iconFrame:SetVertexColor(1, 1, 1, 0.5 - (0.5 * (iconTimer / db.icontime)))
	if iconTimer > db.icontime then
		frame:HideBigIcon()
	end
end

function frame:ShowBigIcon(icon)
	if not iconFrame then
		frame:SetFrameStrata("HIGH")
		frame:SetPoint("CENTER", WorldFrame)
		local width = WorldFrame:GetHeight() / 3
		frame:SetWidth(width)
		frame:SetHeight(width)

		iconFrame = frame:CreateTexture()
		iconFrame:SetAllPoints(frame)
		iconFrame:SetBlendMode("BLEND")
		iconFrame:SetTexCoord(0.09, 0.91, 0.09, 0.91)

		frame:SetScript("OnUpdate", onUpdate)
		frame:Hide()
	end
	iconFrame:SetVertexColor(1, 1, 1, 0.5)
	iconFrame:SetTexture(icon)
	self:Show()
end
function frame:HideBigIcon()
	if not iconFrame then return end
	currentIcon = nil
	iconFrame:SetTexture(1, 0, 0)
	self:Hide()
	iconTimer = 0
end

local menu_options = {
	icon = function(arg)
		if not arg then return " - big icon (" .. (db.icon and "true" or "false") .. ")" end
		db.icon = not db.icon
		print("Huge icon "..(db.icon and "enabled" or "disabled"))
	end,
	raid = function(arg)
		if not arg then return " - raid warning (" .. (db.raid and "true" or "false") .. ")" end
		db.raid = not db.raid
		print("Raid warning "..(db.raid and "enabled" or "disabled"))
	end,
	icontime = function(arg)
		if not arg then return " - icon lingers for (" .. db.icontime .. ")" end
		local newtime = tonumber(arg)
		if newtime then
			db.icontime = newtime
		end
	end,
}
SLASH_BUFFDROP1 = "/buffdrop"
SLASH_BUFFDROP2 = "/bd"
SlashCmdList["BUFFDROP"] = function(arg)
	local command, arguments = string.match(arg, "^(%a+) ?(.*)")
	if menu_options[command] then
		return menu_options[command](arguments)
	end
	print("BuffDrop:")
	for k,v in pairs(menu_options) do
		print(" -"..k..v())
	end
end

